/*******************************************************************************
 * Copyright 2019, 2020 grondag
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 ******************************************************************************/
package grondag.fluidity.base.synch;

import io.netty.buffer.Unpooled;
import org.apiguardian.api.API;
import org.apiguardian.api.API.Status;
import net.minecraft.class_1799;
import net.minecraft.class_2540;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_3222;
import net.minecraft.class_3244;
import net.minecraft.server.MinecraftServer;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.fabricmc.fabric.api.client.networking.v1.ClientPlayNetworking;
import net.fabricmc.fabric.api.networking.v1.PacketSender;

import grondag.fluidity.Fluidity;
import grondag.fluidity.api.article.Article;
import grondag.fluidity.api.storage.Store;

/**
 * Sent when player interacts with the GUI of an IStorage (vs container slots).
 * IStorage has no concept of slots.
 */
@API(status = Status.EXPERIMENTAL)
public class ItemStorageInteractionC2S {
	public static final class_2960 ID = new class_2960(Fluidity.MOD_ID, "posci");

	@Environment(EnvType.CLIENT)
	public static void sendPacket(ItemStorageAction action, DiscreteDisplayDelegate target) {
		if (class_310.method_1551().method_1562() != null) {
			final class_2540 buf = new class_2540(Unpooled.buffer());
			buf.method_10817(action);
			buf.writeInt(target == null ? -1 : target.handle());
			ClientPlayNetworking.send(ID, buf);
		}
	}

	public static void accept(MinecraftServer server, class_3222 player, class_3244 handler, class_2540 buf, PacketSender responseSender) {
		final ItemStorageAction action = buf.method_10818(ItemStorageAction.class);
		final int handle = buf.readInt();

		if (server.method_18854()) {
			acceptInner(action, handle, player);
		} else {
			server.execute(() -> acceptInner(action, handle, player));
		}
	}

	private static void acceptInner(ItemStorageAction action, int handle, class_3222 player) {
		if (player.field_7512 == null || !(player.field_7512 instanceof StorageContainer)) {
			return;
		}

		final Store storage = ((StorageContainer) player.field_7512).getStorage();

		if(storage == null || !storage.isValid()) {
			return;
		}

		final Article targetResource = handle == -1 ? null : storage.view(handle).article();

		switch (action) {
		case PUT_ALL_HELD:
			doPut(false, player, storage);
			return;

		case PUT_ONE_HELD:
			doPut(true, player, storage);
			return;

		case QUICK_MOVE_HALF: {
			if (targetResource == null) {
				return;
			}

			final int toMove = (int) Math.max(1, Math.min(targetResource.toItem().method_7882() / 2, storage.countOf(targetResource)) / 2);
			doQuickMove(toMove, player, targetResource, storage);
			return;
		}

		case QUICK_MOVE_ONE:
			if (targetResource == null) {
				return;
			}
			doQuickMove(1, player, targetResource, storage);
			return;

		case QUICK_MOVE_STACK: {
			if (targetResource == null) {
				return;
			}

			final int toMove = (int) Math.min(targetResource.toItem().method_7882(), storage.countOf(targetResource));
			doQuickMove(toMove, player, targetResource, storage);
			return;
		}

		case TAKE_ONE:
			doTake(1, player, targetResource, storage);
			return;

		case TAKE_HALF: {
			if (targetResource == null) {
				return;
			}

			final int toTake = (int) Math.max(1, Math.min(targetResource.toItem().method_7882() / 2, storage.countOf(targetResource) / 2));
			doTake(toTake, player, targetResource, storage);
			return;
		}

		case TAKE_STACK: {
			if (targetResource == null) {
				return;
			}

			final int toTake = (int) Math.min(targetResource.toItem().method_7882(), storage.countOf(targetResource));
			doTake(toTake, player, targetResource, storage);
			return;
		}

		default:
			return;
		}
	}

	private static void doPut(boolean single, class_3222 player, Store container) {
		final class_1799 cursorStack = player.field_7514.method_7399();

		if (cursorStack != null && !cursorStack.method_7960()) {
			final int added = (int) container.getConsumer().apply(cursorStack, single ? 1 : cursorStack.method_7947(), false);

			if (added > 0){
				cursorStack.method_7934(added);
				player.field_7514.method_7396(cursorStack);
				player.field_7514.method_5431();
				player.method_14241();
			}
		}
		return;
	}

	private static void doQuickMove(int howMany, class_3222 player, Article targetResource, Store listener) {
		if (howMany == 0 || targetResource == null || targetResource.isNothing()) {
			return;
		}

		final int toMove = (int) listener.getSupplier().apply(targetResource, howMany, false);

		if (toMove == 0) {
			return;
		}

		final class_1799 newStack = targetResource.toStack(toMove);
		player.field_7514.method_7398(player.field_6002, newStack);
		player.field_7514.method_5431();
	}

	private static void doTake(int howMany, class_3222 player, Article targetResource, Store container) {
		if (howMany == 0 || targetResource == null || targetResource.isNothing()) {
			return;
		}

		final class_1799 cursorStack = player.field_7514.method_7399();

		if (cursorStack != null && !cursorStack.method_7960()) {
			if (!targetResource.matches(cursorStack)) {
				return;
			}

			if (cursorStack.method_7947() >= cursorStack.method_7914()) {
				return;
			}

			howMany = Math.min(howMany, cursorStack.method_7914() - cursorStack.method_7947());
			final int toAdd = (int) container.getSupplier().apply(targetResource, howMany, false);
			cursorStack.method_7933(toAdd);
			player.field_7514.method_7396(cursorStack);
			player.field_7514.method_5431();
			player.method_14241();
		} else {
			howMany = Math.min(howMany, targetResource.toItem().method_7882());

			final int toAdd = (int) container.getSupplier().apply(targetResource, howMany, false);

			if (toAdd == 0) {
				return;
			}

			final class_1799 newStack = targetResource.toStack(toAdd);
			player.field_7514.method_7396(newStack);
			player.field_7514.method_5431();
			player.method_14241();
		}

	}
}
