/*******************************************************************************
 * Copyright 2019, 2020 grondag
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 ******************************************************************************/
package grondag.fluidity.impl.article;

import grondag.fluidity.api.article.Article;
import grondag.fluidity.api.article.ArticleType;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_2487;
import net.minecraft.class_2520;
import net.minecraft.class_2540;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import org.apiguardian.api.API;
import org.apiguardian.api.API.Status;
import org.jetbrains.annotations.Nullable;

@API(status = Status.INTERNAL)
public class ArticleImpl<T> implements Article {
	final ArticleTypeImpl<T> type;
	final T resource;
	final class_2487 tag;
	final int hashCode;
	String translationKey;

	ArticleImpl(ArticleType<T> type, T resource, @Nullable class_2487 tag) {
		this.type = (ArticleTypeImpl<T>) type;
		this.resource = resource;
		this.tag = tag;

		int hashCode = resource == null ? 0 : resource.hashCode();

		if(tag != null) {
			hashCode += tag.hashCode();
		}

		this.hashCode = hashCode;
	}

	@Override
	public final ArticleType<?> type() {
		return type;
	}

	@SuppressWarnings("unchecked")
	@Override
	public final T resource() {
		return resource;
	}

	@Override
	public final boolean hasTag() {
		return tag != null;
	}

	@Override
	@Nullable
	public final class_2487 copyTag() {
		return tag.method_10553();
	}

	@Override
	public final boolean doesTagMatch(@Nullable class_2487 otherTag) {
		return tag == null ? otherTag == null : tag.equals(otherTag);
	}

	@Override
	public final boolean equals(Object obj) {
		if (obj == this) {
			return true;
		} else if(obj instanceof ArticleImpl) {
			final ArticleImpl<?> other = (ArticleImpl<?>) obj;
			return other.resource == resource && other.type == type && doesTagMatch(other.tag);
		} else {
			return false;
		}
	}

	@Override
	public final int hashCode() {
		return hashCode;
	}

	public static final ArticleImpl<Void> NOTHING = new ArticleImpl<>(ArticleType.NOTHING, null, null);

	@Override
	public class_2520 toTag() {
		final class_2487 result = new class_2487();
		result.method_10566("type", type.toTag());
		result.method_10566("res",type.tagWriter.apply(resource));
		if(this.tag != null) {
			result.method_10566("tag", tag.method_10553());
		}
		return result;
	}

	@Override
	public void toPacket(class_2540 buf) {
		type.toPacket(buf);
		type.packetWriter.accept(resource, buf);
		buf.method_10794(tag);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static Article fromTag(class_2520 tag) {
		if(tag == null) {
			return Article.NOTHING;
		}

		final class_2487 myTag = (class_2487) tag;
		final ArticleTypeImpl type = ArticleTypeImpl.fromTag(myTag.method_10580("type"));
		final Object resource = type.tagReader.apply(myTag.method_10580("res"));
		final class_2487 aTag = myTag.method_10545("tag") ? myTag.method_10562("tag") : null;
		return of(type, resource, aTag);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static Article fromPacket(class_2540 buf) {
		final ArticleTypeImpl type = ArticleTypeImpl.fromPacket(buf);
		final Object resource = type.packetReader.apply(buf);
		final class_2487 aTag = buf.method_10798();
		return of(type, resource, aTag);
	}

	public static <V> Article of(ArticleType<V> type, V resource, @Nullable class_2487 tag) {
		return ArticleCache.getArticle(type, resource, tag);
	}

	public static Article of(class_1792 item) {
		return of(item, null);
	}

	public static Article of(class_1799 stack) {
		return stack.method_7960() ? Article.NOTHING : of(stack.method_7909(), stack.method_7969());
	}

	public static Article of(class_1792 item, @Nullable class_2487 tag) {
		if(item == class_1802.field_8162 || item == null) {
			return NOTHING;
		} else {
			return ArticleCache.getArticle(ArticleType.ITEM, item, tag);
		}
	}

	public static Article of(class_3611 fluid) {
		return fluid == class_3612.field_15906 ? NOTHING : ArticleCache.getArticle(ArticleType.FLUID, fluid, null);
	}

	@Override
	public String getTranslationKey() {
		String result = translationKey;

		if(result == null) {
			result = type.keyFunction.apply(resource);
			translationKey =  result;
		}

		return result;
	}
}
