/*******************************************************************************
 * Copyright 2019, 2020 grondag
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 ******************************************************************************/
package grondag.fluidity.impl.device;

import java.util.IdentityHashMap;
import java.util.function.BiPredicate;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2378;
import net.minecraft.class_2586;
import net.minecraft.class_2680;
import net.minecraft.class_3222;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import org.apache.commons.lang3.tuple.Pair;
import grondag.fluidity.api.device.BlockComponentContext;
import grondag.fluidity.api.device.DeviceComponentAccess;
import grondag.fluidity.api.device.DeviceComponentType;
import grondag.fluidity.api.device.EntityComponentContext;
import grondag.fluidity.api.device.ItemComponentContext;

public final class DeviceComponentTypeImpl<T> implements DeviceComponentType<T>{
	private final T absent;
	private final Function<BlockComponentContext, ?> defaultBlockMapping;
	private final Function<ItemComponentContext, ?> defaultItemMapping;
	private final Function<EntityComponentContext, ?> defaultEntityMapping;

	private final IdentityHashMap<class_2248, Function<BlockComponentContext, ?>> blockMappings = new IdentityHashMap<>();
	private final IdentityHashMap<class_1792, Function<ItemComponentContext, ?>> itemMappings = new IdentityHashMap<>();
	private final IdentityHashMap<class_1299<?>, Function<EntityComponentContext, ?>> entityMappings = new IdentityHashMap<>();
	private final IdentityHashMap<class_1792, ObjectArrayList<BiPredicate<ItemComponentContext, T>>> itemActions = new IdentityHashMap<>();

	private ObjectArrayList<Pair<Function<EntityComponentContext, T>, Predicate<class_1299<?>>>> deferedEntityMappings;

	private final AbsentDeviceComponent<T> absentComponent;

	DeviceComponentTypeImpl(T absent) {
		this.absent = absent;
		defaultBlockMapping = b -> absent;
		defaultItemMapping = i -> absent;
		defaultEntityMapping = e -> absent;
		absentComponent = new AbsentDeviceComponent<>(this);
	}

	@Override
	public T absent() {
		return absent;
	}

	Function<BlockComponentContext, ?> getMapping(class_2248 block) {
		return blockMappings.getOrDefault(block, defaultBlockMapping);
	}

	@Override
	public void registerProvider(Function<BlockComponentContext, T> mapping, class_2248... blocks) {
		for(final class_2248 b : blocks) {
			blockMappings.put(b, mapping);
		}
	}

	Function<EntityComponentContext, ?> getMapping(class_1297 entity) {
		return entityMappings.getOrDefault(entity.method_5864(), defaultEntityMapping);
	}

	@Override
	public void registerProvider(Function<EntityComponentContext, T> mapping, class_1299<?>... entities) {
		for(final class_1299<?> e : entities) {
			entityMappings.put(e, mapping);
		}
	}

	Function<ItemComponentContext, ?> getMapping(class_1792 item) {
		return itemMappings.getOrDefault(item, defaultItemMapping);
	}

	@Override
	public void registerProvider(Function<ItemComponentContext, T> mapping, class_1792... items) {
		for(final class_1792 i : items) {
			itemMappings.put(i, mapping);
		}
	}

	@Override
	public void registerAction(BiPredicate<ItemComponentContext, T> action, class_1792... items) {
		for(final class_1792 i : items) {
			ObjectArrayList<BiPredicate<ItemComponentContext, T>> list = itemActions.get(i);

			if(list == null) {
				list = new ObjectArrayList<>(4);
				itemActions.put(i, list);
			}

			list.add(action);
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public DeviceComponentAccess<T> getAccess(class_1937 world, class_2338 pos) {
		return BlockComponentContextImpl.get(this, world, pos);
	}

	@SuppressWarnings("unchecked")
	@Override
	public DeviceComponentAccess<T> getAccess(class_1937 world, class_2338 pos, class_2680 blockState) {
		return BlockComponentContextImpl.get(this, world, pos, blockState);
	}

	@SuppressWarnings("unchecked")
	@Override
	public DeviceComponentAccess<T> getAccess(class_2586 blockEntity) {
		return BlockComponentContextImpl.get(this, blockEntity);
	}

	@SuppressWarnings("unchecked")
	@Override
	public <E extends class_1297> DeviceComponentAccess<T> getAccess(E entity) {
		applyDeferredEntityRegistrations();
		return EntityComponentContextImpl.get(this, entity);
	}

	@Override
	public DeviceComponentAccess<T> getAbsentAccess() {
		return absentComponent;
	}

	@SuppressWarnings("unchecked")
	@Override
	public DeviceComponentAccess<T> getAccessForHeldItem(Supplier<class_1799> stackGetter, Consumer<class_1799> stackSetter, class_3222 player) {
		return ItemComponentContextImpl.get(this, stackGetter, stackSetter, player);
	}

	@SuppressWarnings("unchecked")
	@Override
	public DeviceComponentAccess<T> getAccess(Supplier<class_1799> stackGetter, Consumer<class_1799> stackSetter, class_1937 world) {
		return ItemComponentContextImpl.get(this, stackGetter, stackSetter, world);
	}

	@Override
	public boolean applyActions(T target, Supplier<class_1799> stackGetter, Consumer<class_1799> stackSetter, class_3222 player) {
		final ObjectArrayList<BiPredicate<ItemComponentContext, T>> actions = itemActions.get(stackGetter.get().method_7909());

		if(actions != null && !actions.isEmpty()) {
			return applyActions(target, actions, ItemComponentContextImpl.get(this, stackGetter, stackSetter, player));
		}

		return false;
	}

	@Override
	public boolean applyActions(T target, Supplier<class_1799> stackGetter, Consumer<class_1799> stackSetter, class_1937 world) {
		final ObjectArrayList<BiPredicate<ItemComponentContext, T>> actions = itemActions.get(stackGetter.get().method_7909());

		if(actions != null && !actions.isEmpty()) {
			return applyActions(target, actions, ItemComponentContextImpl.get(this, stackGetter, stackSetter, world));
		}

		return false;
	}

	private boolean applyActions(T target, ObjectArrayList<BiPredicate<ItemComponentContext, T>> actions, ItemComponentContext ctx) {
		for(final BiPredicate<ItemComponentContext, T> action : actions) {
			if(action.test(ctx, target)) {
				return true;
			}
		}

		return false;
	}

	@Override
	public void registerProvider(Function<EntityComponentContext, T> mapping, Predicate<class_1299<?>> predicate) {
		if(deferedEntityMappings == null) {
			deferedEntityMappings = new ObjectArrayList<>();
		}

		deferedEntityMappings.add(Pair.of(mapping, predicate));
	}

	private void applyDeferredEntityRegistrations() {
		if(deferedEntityMappings != null) {
			for(final Pair<Function<EntityComponentContext, T>, Predicate<class_1299<?>>> pair : deferedEntityMappings) {
				class_2378.field_11145.forEach(e -> {
					if(pair.getRight().test(e)) {
						registerProvider(pair.getLeft(), e);
					}
				});
			}

			deferedEntityMappings = null;
		}
	}
}
