/*******************************************************************************
 * Copyright 2019, 2020 grondag
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 ******************************************************************************/
package grondag.fluidity.wip.base.transport;

import grondag.fluidity.api.article.Article;
import grondag.fluidity.api.fraction.Fraction;
import grondag.fluidity.api.transact.TransactionParticipant;
import grondag.fluidity.wip.api.transport.CarrierSession;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import org.apiguardian.api.API;
import org.apiguardian.api.API.Status;
import org.jetbrains.annotations.Nullable;

/**
 * Mimics consumer/supplier functions - used to throttle throughput.
 * All implementations should be self-enlisting because cost accounting
 * is typically transparent to users of a carrier.
 */
@API(status = Status.EXPERIMENTAL)
public interface CarrierCostFunction extends TransactionParticipant {
	/**
	 * Throttles articles being transported via carrier. May return less than requested.
	 *
	 * @param item Item to add
	 * @param tag NBT if item has it, null otherwise.
	 * @param count How many to add. Must be >= 0;
	 * @param simulate If true, will forecast result without making changes.
	 * @return Count added, or that would be added if {@code simulate} = true.
	 */
	long apply(CarrierSession sender, Article item, long count, boolean simulate);


	default long apply(CarrierSession sender, class_1792 item, @Nullable class_2487 tag, long count, boolean simulate) {
		return apply(sender, Article.of(item, tag), count, simulate);
	}

	default long apply(CarrierSession sender, class_1792 item, long count, boolean simulate) {
		return apply(sender, Article.of(item), count, simulate);
	}

	default long apply(CarrierSession sender, class_1799 stack, long count, boolean simulate) {
		return apply(sender, Article.of(stack), count, simulate);
	}

	default long apply(CarrierSession sender, class_1799 stack, boolean simulate) {
		return apply(sender, Article.of(stack), stack.method_7947(), simulate);
	}

	Fraction apply(CarrierSession sender, Article item, Fraction volume, boolean simulate);

	long apply(CarrierSession sender, Article item, long numerator, long divisor, boolean simulate);

	@Override
	default boolean isSelfEnlisting() {
		return true;
	}

	CarrierCostFunction FREE = new CarrierCostFunction() {
		@Override
		public TransactionDelegate getTransactionDelegate() {
			return TransactionDelegate.IGNORE;
		}

		@Override
		public long apply(CarrierSession sender, Article item, long count, boolean simulate) {
			return count;
		}

		@Override
		public Fraction apply(CarrierSession sender, Article item, Fraction volume, boolean simulate) {
			return volume.toImmutable();
		}

		@Override
		public long apply(CarrierSession sender, Article item, long numerator, long divisor, boolean simulate) {
			return numerator;
		}
	};
}
